unit fWrapper;

interface

uses Classes, ActiveX, OleServer, ServerA_TLB;

type

  TTestFuncOnTimeChange = procedure(Sender: TObject; DateTime: TDateTime) of object;
  TTestFuncOnTextChange = procedure(Sender: TObject; Index: Integer; Text: string) of object;


// *********************************************************************//
// OLE Server Proxy class declaration
// Server Object    : TTestFunc
// Help String      : TestFunc Object
// Default Interface: ITestFunc
// Def. Intf. DISP? : No
// Event   Interface: ITestFuncEvents
// TypeFlags        : (2) CanCreate
// *********************************************************************//
{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
  TTestFuncProperties= class;
{$ENDIF}
  TTestFunc = class(TOleServer)
  private
    FOnTimeChange: TTestFuncOnTimeChange;
    FOnTextChange: TTestFuncOnTextChange;
    FIntf:        ITestFunc;
{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
    FProps:       TTestFuncProperties;
    function      GetServerProperties: TTestFuncProperties;
{$ENDIF}
    function      GetDefaultInterface: ITestFunc;
  protected
    procedure InitServerData; override;
    procedure InvokeEvent(DispID: TDispID; var Params: TVariantArray); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Connect; override;
    procedure ConnectTo(svrIntf: ITestFunc);
    procedure Disconnect; override;
    function  GetTexts(Index: Integer): WideString;
    procedure SetTexts(Index: Integer; const Value: WideString);
    function  GetConnectionCount: Integer;
    property  DefaultInterface: ITestFunc read GetDefaultInterface;
  published
{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
    property Server: TTestFuncProperties read GetServerProperties;
{$ENDIF}
    property OnTimeChange: TTestFuncOnTimeChange read FOnTimeChange write FOnTimeChange;
    property OnTextChange: TTestFuncOnTextChange read FOnTextChange write FOnTextChange;
  end;

{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
// *********************************************************************//
// OLE Server Properties Proxy Class
// Server Object    : TTestFunc
// (This object is used by the IDE's Property Inspector to allow editing
//  of the properties of this server)
// *********************************************************************//
 TTestFuncProperties = class(TPersistent)
  private
    FServer:    TTestFunc;
    function    GetDefaultInterface: ITestFunc;
    constructor Create(AServer: TTestFunc);
  protected
  public
    property DefaultInterface: ITestFunc read GetDefaultInterface;
  published
  end;
{$ENDIF}

implementation

procedure TTestFunc.InitServerData;
const
  CServerData: TServerData = (
    ClassID:   '{4F0ACBBA-EA8E-41E8-A3DA-44849E2FD303}';
    IntfIID:   '{79ADEC3D-1505-407C-ADBE-74AFC3F83AD1}';
    EventIID:  '{DE775A29-29DF-47F0-941A-0DD6EFBD27A8}';
    LicenseKey: nil;
    Version: 500);
begin
  ServerData := @CServerData;
end;

procedure TTestFunc.Connect;
var
  punk: IUnknown;
begin
  if FIntf = nil then
  begin
    punk := GetServer;
    ConnectEvents(punk);
    Fintf:= punk as ITestFunc;
  end;
end;

procedure TTestFunc.ConnectTo(svrIntf: ITestFunc);
begin
  Disconnect;
  FIntf := svrIntf;
  ConnectEvents(FIntf);
end;

procedure TTestFunc.DisConnect;
begin
  if Fintf <> nil then
  begin
    DisconnectEvents(FIntf);
    FIntf := nil;
  end;
end;

function TTestFunc.GetDefaultInterface: ITestFunc;
begin
  if FIntf = nil then
    Connect;
  Assert(FIntf <> nil, 'DefaultInterface is NULL. Component is not connected to Server. You must call ''Connect'' or ''ConnectTo'' before this operation');
  Result := FIntf;
end;

constructor TTestFunc.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
  FProps := TTestFuncProperties.Create(Self);
{$ENDIF}
end;

destructor TTestFunc.Destroy;
begin
{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
  FProps.Free;
{$ENDIF}
  inherited Destroy;
end;

{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
function TTestFunc.GetServerProperties: TTestFuncProperties;
begin
  Result := FProps;
end;
{$ENDIF}

procedure TTestFunc.InvokeEvent(DispID: TDispID; var Params: TVariantArray);
begin
  case DispID of
    -1: Exit;  // DISPID_UNKNOWN
   1: if Assigned(FOnTextChange) then
        FOnTextChange(Self, Params[0], Params[1]);
   2: if Assigned(FOnTimeChange) then
            FOnTimeChange(Self, Params[0] {TDateTime});
  end; {case DispID}
end;

function  TTestFunc.GetTexts(Index: Integer): WideString;
begin
  Result := DefaultInterface.GetTexts(Index);
end;

procedure TTestFunc.SetTexts(Index: Integer; const Value: WideString);
begin
  DefaultInterface.SetTexts(Index, Value);
end;

function  TTestFunc.GetConnectionCount: Integer;
begin
  Result := DefaultInterface.GetConnectionCount;
end;

{$IFDEF LIVE_SERVER_AT_DESIGN_TIME}
constructor TTestFuncProperties.Create(AServer: TTestFunc);
begin
  inherited Create;
  FServer := AServer;
end;

function TTestFuncProperties.GetDefaultInterface: ITestFunc;
begin
  Result := FServer.DefaultInterface;
end;

{$ENDIF}

end.
